import React from "react";
import { View, Text, TextInput, Button } from "react-native";
import { style } from "../util/style";
import { createPost, NewPost, submitLogin } from "../util/api";

export function NewPostContainer({ navigation }): JSX.Element {
  const [currentInput, setCurrentInput] = React.useState<string>("");
  const [currentInputError, setCurrentInputError] = React.useState<string>("");

  return (
    <View style={style.newPostContainer}>
      <TextInput
        style={style.newPostInput}
        placeholder="New Post"
        onChangeText={(text) => {
          setCurrentInput(text);
          setCurrentInputError("");
        }}
        value={currentInput}
      />
      <Button
        title="Post"
        onPress={() => {
          async function submit(): Promise<void> {
            const user = (await submitLogin("demouser", "demopw")) ?? {
              token: "",
            };
            const p: NewPost = {
              content: currentInput,
              token: user.token,
            };
            await createPost(p);
            setCurrentInput("");
          }
          if (currentInput.length > 0) {
            submit();
            navigation.navigate("Home");
          } else {
            setCurrentInputError("Post must not be empty!");
          }
        }}
      />
      <Text style={style.errorText}>{currentInputError}</Text>
    </View>
  );
}
