import React from "react";
import { Button, RefreshControl } from "react-native";
import { getPostsInterval, Post } from "../util/api";
import { PostView } from "./PostView";
import { style } from "../util/style";
import { SafeAreaView } from "react-native";
import { VirtualizedList } from "react-native";

const WINDOW_SIZE = 20;

export function PostsContainer({ navigation }): JSX.Element {
  const [postData, setPostData] = React.useState<Post[]>([]);
  const [offset, setOffset] = React.useState<number>(0);
  const [refreshing, setRefreshing] = React.useState<boolean>(false);

  const handleEndReached = (): void => {
    setOffset(offset + WINDOW_SIZE);
    fetchMorePosts();
  };

  function onRefresh(): void {
    refreshPosts();
  }

  React.useEffect(() => {
    refreshPosts();
  }, []);

  // Full refresh
  function refreshPosts(): void {
    setRefreshing(true);
    setPostData([]);
    setOffset(WINDOW_SIZE);
    async function fetchPosts(): Promise<void> {
      setPostData(await getPostsInterval(0, WINDOW_SIZE));
    }
    fetchPosts();
    setRefreshing(false);
  }

  function fetchMorePosts(): void {
    async function fetchPosts(): Promise<void> {
      setPostData([...postData, ...(await getPostsInterval(offset, WINDOW_SIZE))]);
    }
    fetchPosts();
  }

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const getItemCount = (_data: unknown): number => postData.length;

  const getItem = (_data: unknown, index: number): Post => postData[index];

  return (
    <>
      <NavButton onPress={() => navigation.navigate("New")} text="New Post" />
      <SafeAreaView style={style.postsContainer}>
        <VirtualizedList
          data={postData}
          initialNumToRender={4}
          renderItem={({ item }) => <PostView key={item.id} post={item} />}
          keyExtractor={(item: Post) => item.id}
          getItemCount={getItemCount}
          getItem={getItem}
          onEndReached={handleEndReached}
          onEndReachedThreshold={0.4}
          refreshControl={
            <RefreshControl
              refreshing={refreshing}
              onRefresh={onRefresh}
              colors={["#007AFF"]}
            />
          }
        />
      </SafeAreaView>
    </>
  );
}

function NavButton({
  onPress,
  text,
}: {
  onPress: () => void;
  text: string;
}): JSX.Element {
  return <Button title={text} onPress={onPress} />;
}
